@@ -74,6 +74,7 @@ class Horde_ActiveSync_Device
     const TYPE_WP              = 'windowsphone';
     const TYPE_TOUCHDOWN       = 'touchdown';
     const TYPE_UNKNOWN         = 'unknown';
+    const TYPE_NINE            = 'nine';
 
     /**
      * Device properties.
@@ -103,6 +104,13 @@ class Horde_ActiveSync_Device
      */
     protected $_multiplexSet = false;
 
+    /**
+     * Local override/cache of detected clientType.
+     *
+     * @var string
+     */
+    protected $_clientType;
+
     /**
      * Const'r
      *
@@ -131,13 +139,20 @@ class Horde_ActiveSync_Device
         case self::BLOCKED:
             return $this->_properties['properties'][$property];
         case 'clientType':
-            $type = $this->_getClientType();
-            return $type;
+            if (!isset($this->_clientType)) {
+                $this->_clientType = $this->_getClientType();
+            }
+            return $this->_clientType;
         case self::VERSION:
             if (isset($this->_properties['properties'][self::VERSION])) {
                 return $this->_properties['properties'][self::VERSION];
             }
             break;
+        case self::OS:
+            if (isset($this->_properties['properties'][self::OS])) {
+                return $this->_properties['properties'][self::OS];
+            }
+            break;
         case 'properties':
             if (!isset($this->_properties['properties'])) {
                 $return = array();
@@ -166,6 +181,7 @@ class Horde_ActiveSync_Device
         case self::ANNOUNCED_VERSION:
         case self::BLOCKED:
         case self::VERSION:
+        case self::OS:
             $properties = $this->properties;
             if (empty($properties)) {
                 $properties = array();
@@ -173,6 +189,10 @@ class Horde_ActiveSync_Device
             $properties[$property] = $value;
             $this->setDeviceProperties($properties);
             break;
+        case 'clientType':
+            $this->_clientType = $value;
+            break;
+
         default:
             if (!isset($this->_properties[$property]) || $value != $this->_properties[$property]) {
                 $this->_dirty[$property] = true;
@@ -329,7 +349,7 @@ class Horde_ActiveSync_Device
      * @param boolean $all  If true, save all properties (deviceInfo and
      *                      deviceProperties). Otherwise, just save dirty
      *                      deviceProperties. @since 2.16.0
-     * @todo For 3.0, make it clearer taht deviceInfo is per-user and
+     * @todo For 3.0, make it clearer that deviceInfo is per-user and
      *       deviceProperties is per-device.
      */
     public function save($all = true)
@@ -343,6 +363,12 @@ class Horde_ActiveSync_Device
         $this->_dirty = array();
     }
 
+    /**
+     * Return the major version number of the OS (or client app) as reported
+     * by the client.
+     *
+     * @return integer  The version number.
+     */
     public function getMajorVersion()
     {
         switch (strtolower($this->clientType)) {
@@ -363,6 +389,7 @@ class Horde_ActiveSync_Device
                 }
                 break;
             case self::TYPE_ANDROID:
+            case self::TYPE_NINE:
                 // Most newer Android clients send self::OS, so check that first
                 if (!empty($this->properties[self::OS]) && preg_match('/(\d+)\.(\d+)/', $this->properties[self::OS], $matches)) {
                     return $matches[1];
@@ -386,35 +413,124 @@ class Horde_ActiveSync_Device
         return 0;
     }
 
+    /**
+     * Return the minor version number of the OS (or client app) as reported
+     * by the client.
+     *
+     * @return integer  The version number.
+     */
+    public function getMinorVersion()
+    {
+        switch (strtolower($this->clientType)) {
+            case self::TYPE_BLACKBERRY:
+                if (preg_match('/(.+)\/(.+)/', $this->userAgent, $matches)) {
+                    return $matches[2];
+                }
+                break;
+            case self::TYPE_IPOD:
+            case self::TYPE_IPAD:
+                if (preg_match('/(\d+)\.(\d+)/', $this->properties[self::OS], $matches)) {
+                    return $matches[2];
+                }
+                break;
+            case self::TYPE_IPHONE:
+                if (preg_match('/(.+)\/(\d+)\.(\d+)/', $this->userAgent, $matches)) {
+                    return $matches[3];
+                }
+                break;
+            case self::TYPE_ANDROID:
+                // Most newer Android clients send self::OS, so check that first
+                if (!empty($this->properties[self::OS]) && preg_match('/(\d+)\.(\d+)/', $this->properties[self::OS], $matches)) {
+                    return $matches[2];
+                }
+                // Some newer devices send userAgent like Android/4.3.3-EAS-1.3
+                if (preg_match('/Android\/(\d+)\.(\d+)/', $this->userAgent, $matches)) {
+                    return $matches[2];
+                }
+                // Older Android/0.3 type userAgent strings.
+                if (preg_match('/(.+)\/(\d+)\.(\d+)/', $this->userAgent, $matches)) {
+                    return $matches[3];
+                }
+                break;
+            case self::TYPE_TOUCHDOWN:
+                 if (preg_match('/(.+)\/(\d+)\.(\d+)/', $this->userAgent, $matches)) {
+                    return $matches[3];
+                }
+                break;
+        }
+
+        return 0;
+    }
+
     /**
      * Return the number of hours to offset a POOMCONTACTS:BIRTHDAY
      * or ANNIVERSARY field in an attempt to work around a bug in the
      * protocol - which doesn't define a standard time for birthdays to occur.
      *
-     * @param Horde_Date $date  The date.
-     * @param boolean $toEas    Convert from local to device if true.
+     *  WP:
+     *     Devices seem to send the birthdays at the entered date, with
+     *     a time of 00:00:00 UTC during standard time and with 01:00:00 UTC
+     *     during DST if the client's configured timezone observes it. No idea
+     *     what purpose this serves since no timezone data is transmitted for
+     *     birthday values.
      *
-     * @return Horde_Date  The date of the birthday/anniversary, in UTC, with
-     *                     any fixes applied for the current device.
+     *   iOS:
+     *     Seems different based on version. iOS 5+, at least seems to send
+     *     the birthday as midnight at the entered date in the device's timezone
+     *     then converted to UTC. Some minor issues with offsets being off an
+     *     hour or two for some timezones though.
+     *
+     *     iOS < 5 sends the birthday time part as the time the birthday
+     *     was entered/edited on the device, converted to UTC, so it can't be
+     *     trusted at all. The best we can do here is transform the date to
+     *     midnight on date_default_timezone() converted to UTC.
+     *
+     *   Android:
+     *     For contacts originating on the SERVER, the following is true:
+     *
+     *     Stock 4.3 Takes the down-synched bday value which is assumed to be
+     *     UTC, does some magic to it (converts to milliseconds, creates a
+     *     gregorian calendar object, then converts to YYYY-MM-DD). When
+     *     sending the bday value up, it sends it up as-is. No conversion
+     *     to/from UTC or local is done.
+     *
+     *     Stock 4.4.x does the above, but before sending the bday value,
+     *     validates that it's in a correct format for sending to the server.
+     *     This really only affects date data originally entered on the device
+     *     for non-stock android clients.
+     *
+     *     There is some strange bit of code in Android that adds 1 to the
+     *     DAY_OF_MONTH when HOUR_OF_DAY >= 12 in an attempt to "fix"
+     *     birthday handling for GMT+n users. See:
+     *     https://android.googlesource.com/platform/packages/apps/Exchange/+/32daacdd71b9de8fd5e3f59c37934e3e4a9fa972%5E!/exchange2/src/com/android/exchange/adapter/ContactsSyncAdapter.java
+     *     Not sure what to make of it, or why it's not just converted to
+     *     local tz when displaying but this probably breaks birthday handling
+     *     for people in a few timezones.
+     *
+     *     For contacts originating on the CLIENT, the datetime is sent as
+     *     08:00:00 UTC, and this seems to be regardless of the timezone set
+     *     in the Android system.
+     *
+     *     Given all of this, it makes sense to me to ALWAYS send birthday
+     *     data as occuring at 08:00:00 UTC for *native* Android clients.
+     *
+     *   BB 10+ expects it at 12:00:00 UTC
+     *
+     * @param Horde_Date $date  The date. This should normally be in the local
+     *                          timezone if encoding the date for the client.
+     *                          If decoding the date from the client, it will
+     *                          normally be in UTC.
+     * @param boolean $toEas    Convert from local to device if true.
+     *                          DEFAULT: false
+     *
+     * @return Horde_Date  The date of the birthday/anniversary, with
+     *                     any fixes applied for the current device. The
+     *                     timezone set in the object will depend on the
+     *                     client detected, and whether the date is being
+     *                     encoding or decoding.
      */
     public function normalizePoomContactsDates($date, $toEas = false)
     {
-        // WP devices seem to send the birthdays at the entered date, with
-        // a time of 00:00:00 UTC.
-        //
-        // iOS seems different based on version. iOS 5+, at least seems to send
-        // the birthday as midnight at the entered date in the device's timezone
-        // then converted to UTC. Some minor issues with offsets being off an
-        // hour or two for some timezones though.
-        //
-        // iOS < 5 sends the birthday time part as the time the birthday
-        // was entered/edited on the device, converted to UTC, so it can't be
-        // trusted at all. The best we can do here is transform the date to
-        // midnight on date_default_timezone() converted to UTC.
-        //
-        // Native Android 4 ALWAYS sends it as 08:00:00 UTC
-        //
-        // BB 10+ expects it at 12:00:00 UTC
         switch (strtolower($this->clientType)) {
         case self::TYPE_WP:
         case 'wp8': // Legacy. Remove in H6.
@@ -422,7 +538,8 @@ class Horde_ActiveSync_Device
             if ($toEas) {
                 return new Horde_Date($date->format('Y-m-d'), 'UTC');
             } else {
-                return new Horde_Date($date->format('Y-m-d'));
+                $date = new Horde_Date($date->format('Y-m-d'));
+                return $date->setTimezone('UTC');
             }
 
         case self::TYPE_ANDROID:
@@ -430,7 +547,8 @@ class Horde_ActiveSync_Device
                 if ($toEas) {
                     return new Horde_Date($date->format('Y-m-d 08:00:00'), 'UTC');
                 } else {
-                    return new Horde_Date($date->format('Y-m-d'));
+                    $date = new Horde_Date($date->format('Y-m-d'));
+                    return $date->setTimezone('UTC');
                 }
             } else {
                 // POOMCONTACTS:BIRTHDAY not really supported in early Android
@@ -443,7 +561,11 @@ class Horde_ActiveSync_Device
         case self::TYPE_IPOD:
             if ($this->getMajorVersion() >= 5) {
                 // iOS >= 5 handles it correctly more or less.
-                return $date;
+                if ($toEas) {
+                    return new Horde_Date($date->format('Y-m-d 00:00:00'));
+                } else {
+                    return $date;
+                }
             } else {
                 if ($toEas) {
                     return new Horde_Date($date->format('Y-m-d'), 'UTC');
@@ -452,11 +574,19 @@ class Horde_ActiveSync_Device
                 }
             }
 
+        case self::TYPE_NINE:
+                if ($toEas) {
+                    return new Horde_Date($date->format('Y-m-d 00:00:00'));
+                } else {
+                    return $date;
+                }
+
         case self::TYPE_BLACKBERRY:
             if ($toEas) {
                 return new Horde_Date($date->format('Y-m-d 12:00:00'), 'UTC');
             } else {
-                return new Horde_Date($date->format('Y-m-d'));
+                $date = new Horde_Date($date->format('Y-m-d'));
+                return $date->setTimezone('UTC');
             }
 
         case self::TYPE_TOUCHDOWN:
@@ -470,8 +600,7 @@ class Horde_ActiveSync_Device
      * Attempt to determine the *client* application as opposed to the device,
      * which may or may not be the client.
      *
-     * @return string  The client name, or self::TYPE_UNKNOWN if unable to
-     *                 determine.
+     * @return string  The client name.
      */
     protected function _getClientType()
     {
@@ -480,20 +609,36 @@ class Horde_ActiveSync_Device
              stripos($this->properties[self::OS], 'Android') !== false) ||
              strtolower($this->deviceType) == self::TYPE_ANDROID) {
 
-            // We can detect native android and TouchDown so far.
+            // We can detect native Android, TouchDown, and Nine.
             // Moxier does not distinguish itself, so we can't sniff it.
             if (strpos($this->userAgent, 'TouchDown') !== false) {
                 return self::TYPE_TOUCHDOWN;
+            } else if ($this->_isNine()) {
+                return self::TYPE_NINE;
             } else if (stripos($this->userAgent, 'Android') !== false) {
                 return $this->deviceType;
             } else {
-                return self::TYPE_UNKNOWN;
+                return self::TYPE_ANDROID;
             }
        } else {
             return $this->deviceType;
        }
     }
 
+    /**
+     * Helper method to sniff out the 9Folders client, "Nine".
+     * @see https://ninefolders.plan.io/track/7048/46b213 for the discussion on
+     * how to sniff out the Nine client. Not the best solution, but it's the one
+     * they decided to use.
+     *
+     * @return boolean  True if client is thought to be "Nine".
+     */
+    protected function _isNine()
+    {
+        return stripos(pack('H*', $this->id), 'nine') === 0;
+    }
+
+
     /**
      * Basic sniffing for determining if devices can support non-multiplexed
      * collections.
@@ -505,6 +650,18 @@ class Horde_ActiveSync_Device
             // iOS seems to support multiple collections for everything except Notes.
             $this->_properties['properties'][self::MULTIPLEX] = Horde_ActiveSync_Device::MULTIPLEX_NOTES;
         } else if ($clientType == self::TYPE_ANDROID) {
+            // Special cases: Samsung's native EAS client doesn't support
+            // multiplex.
+            if (stripos($this->deviceType, 'samsung') === 0) {
+                $this->_properties['properties'][self::MULTIPLEX] =
+                    Horde_ActiveSync_Device::MULTIPLEX_CONTACTS |
+                    Horde_ActiveSync_Device::MULTIPLEX_CALENDAR |
+                    Horde_ActiveSync_Device::MULTIPLEX_NOTES |
+                    Horde_ActiveSync_Device::MULTIPLEX_TASKS;
+
+                return;
+            }
+
             // All android before 4.4 KitKat requires multiplex. KitKat supports
             // non-multiplexed calendars only.
             if (!empty($this->properties[self::OS]) &&
