@@ -139,6 +139,25 @@ class Horde_ActiveSync_Imap_Adapter
         }
     }
 
+    /**
+     * Completely empty specified mailbox.
+     *
+     * @param string $mbox  The mailbox to empty.
+     *
+     * @throws Horde_ActiveSync_Exception
+     * @since 2.18.0
+     */
+    public function emptyMailbox($mbox)
+    {
+        $imap = $this->_getImapOb();
+        $mbox = new Horde_Imap_Client_Mailbox($mbox);
+        try {
+            $imap->expunge($mbox, array('delete' => true));
+        } catch (Horde_Imap_Client_Exception $e) {
+            throw new Horde_ActiveSync_Exception($e);
+        }
+    }
+
     /**
      * Permanently delete a mail message.
      *
@@ -298,6 +317,7 @@ class Horde_ActiveSync_Imap_Adapter
             serialize($status))
         );
         $flags = array();
+        $categories = array();
         $modseq = $status[Horde_ActiveSync_Folder_Imap::HIGHESTMODSEQ];
         if ($modseq && $folder->modseq() > 0 && $folder->modseq() < $modseq) {
             $this->_logger->info(sprintf(
@@ -320,6 +340,10 @@ class Horde_ActiveSync_Imap_Adapter
             // $modseq sneak in yet (they will be caught on the next PING or
             // SYNC).
             $changes = array();
+
+            // Get custom flags to use as categories.
+            $msgFlags = $this->_getMsgFlags();
+
             foreach ($fetch_ret as $uid => $data) {
                 if ($options['sincedate']) {
                     $since = new Horde_Date($options['sincedate']);
@@ -345,9 +369,17 @@ class Horde_ActiveSync_Imap_Adapter
                     if (($options['protocolversion']) > Horde_ActiveSync::VERSION_TWOFIVE) {
                         $flags[$uid]['flagged'] = (array_search(Horde_Imap_Client::FLAG_FLAGGED, $data->getFlags()) !== false) ? 1 : 0;
                     }
+                    if ($options['protocolversion'] > Horde_ActiveSync::VERSION_TWELVEONE) {
+                        $categories[$uid] = array();
+                        foreach ($data->getFlags() as $flag) {
+                            if (!empty($msgFlags[strtolower($flag)])) {
+                                $categories[$uid][] = $msgFlags[strtolower($flag)];
+                            }
+                        }
+                    }
                 }
             }
-            $folder->setChanges($changes, $flags);
+            $folder->setChanges($changes, $flags, $categories);
             try {
                 $deleted = $imap->vanished(
                     $mbox,
@@ -538,6 +570,8 @@ class Horde_ActiveSync_Imap_Adapter
      *
      * @return array  An array with the 0 element being the mbox
      * @throws Horde_Exception_NotFound, Horde_ActiveSync_Exception
+     *
+     * @deprecated This is unused and should be removed.
      */
     public function getUidFromMidInFolders($id, array $folders)
     {
@@ -771,6 +805,31 @@ class Horde_ActiveSync_Imap_Adapter
         }
     }
 
+    public function categoriesToFlags($mailbox, $categories, $uid)
+    {
+        $msgFlags = $this->_getMsgFlags();
+        $mbox = new Horde_Imap_Client_Mailbox($mailbox);
+        $options = array(
+            'ids' => new Horde_Imap_Client_Ids(array($uid)),
+            'add' => array()
+        );
+        foreach ($categories as $category) {
+            // Do our best to make sure the imap flag is a RFC 3501 compliant.
+            $atom = new Horde_Imap_Client_Data_Format_Atom(strtr(Horde_String_Transliterate::toAscii($category), ' ', '_'));
+            $imapflag = strtolower($atom->stripNonAtomCharacters());
+            if (!empty($msgFlags[$imapflag])) {
+                $options['add'][] = $imapflag;
+                unset($msgFlags[$imapflag]);
+            }
+        }
+        $options['remove'] = array_keys($msgFlags);
+        try {
+            $this->_getImapOb()->store($mbox, $options);
+        } catch (Horde_Imap_Client_Exception $e) {
+            throw new Horde_ActiveSync_Exception($e);
+        }
+    }
+
     /**
      * Set the message's read status.
      *
@@ -1047,6 +1106,17 @@ class Horde_ActiveSync_Imap_Adapter
                 }
                 $eas_message->airsyncbaseattachments = $imap_message->getAttachments($version);
             }
+
+            if ($version > Horde_ActiveSync::VERSION_TWELVEONE) {
+                $flags = array();
+                $msgFlags = $this->_getMsgFlags();
+                foreach ($imap_message->getFlags() as $flag) {
+                    if (!empty($msgFlags[strtolower($flag)])) {
+                        $flags[] = $msgFlags[strtolower($flag)];
+                    }
+                }
+                $eas_message->categories = $flags;
+            }
         }
 
         // Preview?
@@ -1484,4 +1554,13 @@ class Horde_ActiveSync_Imap_Adapter
         return $data;
     }
 
+    protected function _getMsgFlags()
+    {
+        // @todo Horde_ActiveSync 3.0 remove method_exists check.
+        if (method_exists($this->_imap, 'getMsgFlags')) {
+            return $this->_imap->getMsgFlags();
+        }
+
+        return array();
+    }
 }
