@@ -22,6 +22,7 @@
  * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
+ * @internal
  */
 abstract class Horde_ActiveSync_Request_Base
 {
@@ -244,7 +245,10 @@ abstract class Horde_ActiveSync_Request_Base
             $this->_device->version,
             Horde_ActiveSync::LIBRARY_VERSION)
         );
-
+        $this->_logger->info(sprintf(
+            '[%s] GET VARIABLES: %s',
+            $this->_procid,
+            print_r($this->_activeSync->getGetVars(), true)));
         try {
             return $this->_handle();
         } catch (Exception $e) {
