@@ -74,18 +74,29 @@ class Horde_ActiveSync_Rfc822
     public function __construct($rfc822, $auto_add_headers = true)
     {
         if (is_resource($rfc822)) {
-            $this->_stream = new Horde_Stream_Existing(array('stream' => $rfc822));
-            rewind($this->_stream->stream);
+            $stream = new Horde_Stream_Existing(array('stream' => $rfc822));
+            $stream->rewind();
         } else {
-            $this->_stream = new Horde_Stream_Temp(array('max_memory' => self::$memoryLimit));
-            $this->_stream->add($rfc822, true);
+            $stream = new Horde_Stream_Temp(array('max_memory' => self::$memoryLimit));
+            $stream->add($rfc822, true);
         }
-        list($this->_hdr_pos, $this->_eol) = $this->_findHeader();
+        $this->_parseStream($stream);
         if ($auto_add_headers) {
             $this->addStandardHeaders();
         }
     }
 
+    /**
+     * Parse a Horde_Stream object to get the header and eol data.
+     *
+     * @param  Horde_Stream $stream  The stream object.
+     */
+    protected function _parseStream(Horde_Stream $stream)
+    {
+        $this->_stream = $stream;
+        list($this->_hdr_pos, $this->_eol) = $this->_findHeader();
+    }
+
     /**
      * Returns the raw message with the message headers stripped.
      *
@@ -100,6 +111,29 @@ class Horde_ActiveSync_Rfc822
         return $new_stream;
     }
 
+    /**
+     * Replace the MIME part of the message sent from the client.
+     *
+     * @param  Horde_Mime_Part $part  The new MIME part.
+     * @since 2.19.0
+     */
+    public function replaceMime(Horde_Mime_Part $part)
+    {
+        $mime_stream = $part->toString(array(
+            'stream' => true,
+            'headers' => false)
+        );
+
+        if (!empty($this->_header_text)) {
+            $hdr = $this->_header_text;
+        } else {
+            $this->_stream->rewind();
+            $hdr = $this->_stream->substring(0, $this->_hdr_pos);
+        }
+        $new_stream = Horde_Stream_Wrapper_Combine::getStream(array($hdr, $mime_stream));
+        $this->_parseStream(new Horde_Stream_Existing(array('stream' => $new_stream)));
+    }
+
     /**
      * Return the raw message data.
      *
@@ -150,7 +184,7 @@ class Horde_ActiveSync_Rfc822
             $updated = true;
         }
         if ($updated) {
-            $this->_header_text = $headers->toString();
+            $this->_header_text = $headers->toString(array('charset' => 'UTF-8'));
         }
     }
 
