@@ -682,6 +682,18 @@ class Horde_ActiveSync_SyncCache
         $this->_dirty['folders'] = true;
     }
 
+    /**
+     * Refresh the folder cache from the backend.
+     *
+     * @since 2.18.0
+     */
+    public function refreshFolderCache()
+    {
+        $cache = $this->_state->getSyncCache($this->_devid, $this->_user);
+        $this->_data['folders'] = $cache['folders'];
+        $this->_dirty['folders'] = false;
+    }
+
     /**
      * Update a folder entry in the cache.
      *
