@@ -206,4 +206,65 @@ class Horde_ActiveSync_Utils
         return base64_encode($goid);
     }
 
+    /**
+     * Ensure $data is converted to valid UTF-8 data. Works as follows:
+     * Converts to UTF-8, assuming data is in $from_charset encoding. If
+     * that produces invalid UTF-8, attempt to convert to most common mulitibyte
+     * encodings. If that *still* fails, strip out non 7-Bit characters...and
+     * force encoding to UTF-8 from $from_charset as a last resort.
+     *
+     * @param string $data          The string data to convert to UTF-8.
+     * @param string $from_charset  The character set to assume $data is encoded
+     *                              in.
+     *
+     * @return string  A valid UTF-8 encoded string.
+     */
+    static public function ensureUtf8($data, $from_charset)
+    {
+        $text = Horde_String::convertCharset($data, $from_charset, 'UTF-8');
+        if (!Horde_String::validUtf8($text)) {
+            $test_charsets = array(
+                'windows-1252',
+                'UTF-8'
+            );
+            foreach ($test_charsets as $charset) {
+                if ($charset != $from_charset) {
+                    $text = Horde_String::convertCharset($data, $charset, 'UTF-8');
+                    if (Horde_String::validUtf8($text)) {
+                        return $text;
+                    }
+                }
+            }
+            // Invalid UTF-8 still found. Strip out non 7-bit characters, or if
+            // that fails, force a conersion to UTF-8 as a last resort. Need
+            // to break string into smaller chunks to avoid hitting
+            // https://bugs.php.net/bug.php?id=37793
+            $chunk_size = 4000;
+            $text = '';
+            while ($data !== false && strlen($data)) {
+                $test = self::_stripNon7BitChars(substr($data, 0, $chunk_size));
+                if ($test !== false) {
+                    $text .= $test;
+                } else {
+                    return Horde_String::convertCharset($data, $from_charset, 'UTF-8', true);
+                }
+                $data = substr($data, $chunk_size);
+            }
+        }
+
+        return $text;
+    }
+
+    /**
+     * Strip out non 7Bit characters from a text string.
+     *
+     * @param string $text  The string to strip.
+     *
+     * @return string|boolean  The stripped string, or false if failed.
+     */
+    static protected function _stripNon7BitChars($text)
+    {
+        return preg_replace('/[^\x09\x0A\x0D\x20-\x7E]/', '', $text);
+    }
+
 }
\ No newline at end of file
